/*! 
  \file fff_glm_mfx.h
  \brief General linear model under observation errors (mixed effects) 
  \author Alexis Roche
  \date 2008
  
  Bla bla bla

  */


#ifndef FFF_GLM_MFX
#define FFF_GLM_MFX

#ifdef __cplusplus
extern "C" {
#endif

#include "fff_vector.h"
#include "fff_matrix.h"
  
  /*! 
    \brief Multiple regression under mixed effects using the EM algorithm
    \param b final effect estimate
    \param s2 final 2nd level variance estimate 
    \param z expected true effects
    \param vz expected variance of the true effects (diagonal matrix)
    \param Qz expected prediction error
    \param y observations
    \param vy 1st level variances 
    \param X design matrix 
    \param PpiX pre-computed projected pseudo inverse of X
    \param niter number of EM iterations

    Returns the final log-likelihood.

    The function requires the projected pseudo-inverse matrix PpiX to
    be pre-calculated externally. It is defined by: \f$ PpiX = P
    (X'X)^{-1} X' \f$, where: \f$ P = I_p - A C (C' A C)^{-1} C' \f$
    with \f$ A = (X'X)^-1 \f$ is the appropriate projector onto the
    constaint space, \f$ C'b=0 \f$. \a P is, in fact, orthogonal for
    the dot product defined by \a X'X.
    
    Please note that the equality \a PpiX*X=P should hold but is not
    checked.
  */
  extern void  fff_glm_mfx_EM( fff_vector* b, 
			       double* s2, 
			       fff_vector* z, 
			       fff_vector* vz,
			       fff_vector* Qz,
			       const fff_vector* y, 
			       const fff_vector* vy, 
			       const fff_matrix* X, 
			       const fff_matrix* PpiX, 
			       unsigned int niter );
  extern double fff_glm_mfx_log_likelihood( const fff_vector* y, 
					    const fff_vector* vy, 
					    const fff_matrix* X, 
					    const fff_vector* b, 
					    double s2,
					    fff_vector* tmp ); 
  

#ifdef __cplusplus
}
#endif

#endif
