# -*- Mode: Python -*-  Not really, but the syntax is close enough

"""
Python access to core fff functions written in C. This module is
mainly used for unitary tests.

Author: Alexis Roche, 2008.
"""

__version__ = '0.1'

# Includes from the python headers
include "numpy.pxi"

# Redefine size_t
ctypedef unsigned long int size_t

cdef extern from "fff_base.h":

    ctypedef enum fff_datatype:
        FFF_UNKNOWN_TYPE = -1, 
        FFF_UCHAR = 0,         
        FFF_SCHAR = 1,         
        FFF_USHORT = 2,        
        FFF_SSHORT = 3,        
        FFF_UINT = 4,          
        FFF_INT = 5,           
        FFF_ULONG = 6,         
        FFF_LONG = 7,          
        FFF_FLOAT = 8,         
        FFF_DOUBLE = 9         


cdef extern from "fff_array.h":

    ctypedef enum fff_array_ndims: 
        FFF_ARRAY_1D = 1,   
        FFF_ARRAY_2D = 2,   
        FFF_ARRAY_3D = 3,   
        FFF_ARRAY_4D = 4    
        
    ctypedef struct fff_array:
        fff_array_ndims ndims
        fff_datatype datatype
        size_t dimX
        size_t dimY
        size_t dimZ
        size_t dimT
        size_t offsetX
        size_t offsetY
        size_t offsetZ
        size_t offsetT
        void* data
        int owner

    fff_array* fff_array_new( fff_datatype datatype, size_t dimX, size_t dimY, size_t dimZ, size_t dimT )
    void fff_array_delete( fff_array* thisone )
    double fff_array_get( fff_array* thisone, size_t x, size_t y, size_t z, size_t t )
    fff_array fff_array_get_block( fff_array* thisone,
                                   size_t x0, size_t x1, size_t fX,
                                   size_t y0, size_t y1, size_t fY,
                                   size_t z0, size_t z1, size_t fZ,
                                   size_t t0, size_t t1, size_t fT )
    void fff_array_set( fff_array* thisone, size_t x, size_t y, size_t z, size_t t, double value )
    void fff_array_set_all( fff_array* thisone, double c )
    void fff_array_extrema ( double* min, double* max, fff_array* thisone )
    void fff_array_copy( fff_array* ares,  fff_array* asrc )
    void fff_array_add (fff_array * x, fff_array * y)
    void fff_array_sub (fff_array * x, fff_array * y)
    void fff_array_div (fff_array * x, fff_array * y)
    void fff_array_mul (fff_array * x, fff_array * y)
    void fff_array_clamp( fff_array* ares, fff_array* asrc, double th, int* clamp )
    

    
# Export the Python fff wrapper
cdef extern from "fffpy.h":

    void fffpy_import_array()
    fff_array* fff_array_fromPyArray( ndarray x )
    ndarray fff_array_toPyArray( fff_array* y )


# Initialize numpy
fffpy_import_array()
import_array()
import numpy as np

# Binded routines
def array_get(A, size_t x, size_t y=0, size_t z=0, size_t t=0):
    """
    Get array element.
    va = array_get(A, size_t x, size_t y=0, size_t z=0, size_t t=0):
    """
    cdef fff_array* a
    cdef double va
    a = fff_array_fromPyArray(A)
    va = fff_array_get(a, x, y, z, t)
    fff_array_delete(a)
    return va

def array_get_block( A, size_t x0, size_t x1, size_t fX=1,
                     size_t y0=0, size_t y1=0, size_t fY=1,
                     size_t z0=0, size_t z1=0, size_t fZ=1,
                     size_t t0=0, size_t t1=0, size_t fT=1 ):
    """
    Get block
    Asub = array_get_block( A, size_t x0, size_t x1, size_t fX=1,
                               size_t y0=0, size_t y1=0, size_t fY=1,
                               size_t z0=0, size_t z1=0, size_t fZ=1,
                               size_t t0=0, size_t t1=0, size_t fT=1 )
    """
    cdef fff_array *a, *b
    cdef fff_array asub
    a = fff_array_fromPyArray(A)
    asub = fff_array_get_block(a, x0, x1, fX, y0, y1, fY, z0, z1, fZ, t0, t1, fT)
    b = fff_array_new(asub.datatype, asub.dimX, asub.dimY, asub.dimZ, asub.dimT)
    fff_array_copy(b, &asub)
    B = fff_array_toPyArray(b)
    fff_array_delete(a)
    return B


